#include "keyboard.h"
#include "codes.hpp"

#define CODES_SPECIALS 11
const unsigned char lookup_specials[2*CODES_SPECIALS] =
{
    'n', KEY_ENTER,
    't', KEY_TAB,
    'b', KEY_BACKSPACE,
    'e', KEY_ESC,
    'l', KEY_CAPSLOCK,
    '\\', KEY_BACKSLASH,
    
    'p', KEY_ERR_OVF, // pause

    's', 0x80|KEY_MOD_LSHIFT,
    'c', 0x80|KEY_MOD_LCTRL,
    'a', 0x80|KEY_MOD_LALT,
    'g', 0x80|KEY_MOD_LMETA
};

const unsigned char lookup_en[96] =
{
    KEY_SPACE, // space
    0x40|KEY_1, // !
    0x40|KEY_APOSTROPHE, // "
    0x40|KEY_3, // #
    0x40|KEY_4, // $
    0x40|KEY_5, // %
    0x40|KEY_7, // &
    KEY_APOSTROPHE, // '
    0x40|KEY_9, // (
    0x40|KEY_0, // )
    0x40|KEY_8, // *
    0x40|KEY_EQUAL, // +
    KEY_COMMA, // ,
    KEY_MINUS, // -
    KEY_DOT, // .
    KEY_SLASH, // /
    KEY_0, // 0
    KEY_1, // 1
    KEY_2, // 2
    KEY_3, // 3
    KEY_4, // 4
    KEY_5, // 5
    KEY_6, // 6
    KEY_7, // 7
    KEY_8, // 8
    KEY_9, // 9
    0x40|KEY_SEMICOLON, // :
    KEY_SEMICOLON, // ;
    0x40|KEY_COMMA, // <
    KEY_EQUAL, // =
    0x40|KEY_DOT, // >
    0x40|KEY_SLASH, // ?
    0x40|KEY_2, // @
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    KEY_LEFTBRACE, // [
    KEY_BACKSLASH, //
    KEY_RIGHTBRACE, // ]
    0x40|KEY_6, // ^
    0x40|KEY_MINUS, // _
    KEY_GRAVE, // `
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0x40|KEY_LEFTBRACE, // {
    0x40|KEY_BACKSLASH, // |
    0x40|KEY_RIGHTBRACE, // }
    0x40|KEY_GRAVE, // ~
    0
};

const unsigned char lookup_cz[96] =
{
    KEY_SPACE, // space
    0x40|KEY_APOSTROPHE, // !
    0x40|KEY_SEMICOLON, // "
    0, // #
    0, // $
    0x40|KEY_MINUS, // %
    0, // &
    0, // '
    0x40|KEY_RIGHTBRACE, // (
    KEY_RIGHTBRACE, // )
    0, // *
    KEY_1, // +
    KEY_COMMA, // ,
    KEY_SLASH, // -
    KEY_DOT, // .
    0x40|KEY_LEFTBRACE, // /
    0x40|KEY_0, // 0
    0x40|KEY_1, // 1
    0x40|KEY_2, // 2
    0x40|KEY_3, // 3
    0x40|KEY_4, // 4
    0x40|KEY_5, // 5
    0x40|KEY_6, // 6
    0x40|KEY_7, // 7
    0x40|KEY_8, // 8
    0x40|KEY_9, // 9
    0x40|KEY_DOT, // :
    0, // ;
    0, // <
    KEY_MINUS, // =
    0, // >
    0x40|KEY_COMMA, // ?
    0, // @
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, // [
    KEY_GRAVE, // backslash
    0, // ]
    0, // ^
    0x40|KEY_SLASH, // _
    0x40|KEY_BACKSLASH, // `
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, // {
    0x40|KEY_GRAVE, // |
    0, // }
    0, // ~
    0
};

QByteArray strToKeys(const QString &s, bool cz)
{
    QByteArray b;
    int special = 0, code = 0;
    for (int z = 0; z < s.size(); ++z) {
        QChar q = s.at(z);
        unsigned char c = q.toLatin1();
        if (special) {
            if (!(--special)) {
                bool found = false;
                for (int i = 0; i < CODES_SPECIALS && !found; ++i) {
                    if (c == lookup_specials[2*i]) {
                        b.append(lookup_specials[2*i+1]);
                        found = true;
                    }
                }
                if (c == 'd') {
                    b.append(0xff);
                    b.append(KEY_DELETE);
                } else if (c == 'x') {
                    special = 10;
                    code = 0;
                } else if (c == 'm') {
                    special = 20;
                    code = 0;
                } else if (!found)
                    return QByteArray();
            } else {
                if (c >= '0' && c <= '9') {
                    code = 16*code + c - '0';
                } else if (c >= 'a' && c <= 'f') {
                    code = 16*code + 10 + c - 'a';
                } else if (c >= 'A' && c <= 'F') {
                    code = 16*code + 10 + c - 'A';
                } else
                    return QByteArray();
                if (special%10 <= 8) {
                    if (special == 8) {
                        b.append(0xff);
                        b.append(code);
                    } else if (special == 18) {
                        if ((0x80|code) == 0xff) return QByteArray();
                        b.append(0x80|code);
                    } else
                        return QByteArray();
                    special = 0;
                }
            }
        } else if (c == '\\') {
            special = 1;
        } else if (c >= 'a' and c <= 'z') {
            if (cz && c == 'y') b.append(KEY_Z);
            else if (cz && c == 'z') b.append(KEY_Y);
            else b.append(KEY_A+c-'a');
        } else if (c >= 'A' and c <= 'Z') {
            if (cz && c == 'Y') b.append(0x40|KEY_Z);
            else if (cz && c == 'Z') b.append(0x40|KEY_Y);
            else b.append(0x40|(KEY_A+c-'A'));
        } else if (c >= 32 && c < 128 &&
                   ((cz && lookup_cz[c-32]) || (!cz && lookup_en[c-32]))) {
            b.append(cz ? lookup_cz[c-32] : lookup_en[c-32]);
        } else
            return QByteArray();
    }
    if (special)
        return QByteArray();
    return b;
}

QString keysToStr(const QByteArray &b, bool cz)
{
    QString s;
    bool special = false;
    for (int z = 0; z < b.size(); ++z) {
        unsigned char c = b.at(z);
        if (special) {
            special = false;
            if (c != KEY_DELETE) {
                QString a;
                a.sprintf("\\x%x%x", c/16, c%16);
                s += a;
            } else
                s += "\\d";
        } else {
            if (c >= 0x80) {
                if (c == 0xff) {
                    special = true;
                } else {
                    bool found = false;
                    for(int i = 0; i < CODES_SPECIALS && !found; ++i) {
                        if (c == lookup_specials[2*i+1]) {
                            s += '\\';
                            s += lookup_specials[2*i];
                            found = true;
                        }
                    }
                    if (!found) {
                        QString a;
                        a.sprintf("\\m%x%x", (c&0x7f)/16, c%16);
                        s += a;
                    }
                }
            } else if ((c&0x3f) >= KEY_A && (c&0x3f) <= KEY_Z) {
                if (cz && (c&0x3f) == KEY_Y) c = KEY_Z+(c&0x40);
                else if (cz && (c&0x3f) == KEY_Z) c = KEY_Y+(c&0x40);
                if (c&0x40) s += ((c&0x3f)+'A'-KEY_A);
                else s += (c+'a'-KEY_A);
            } else {
                bool found = false;
                for(int i = 0; i < CODES_SPECIALS && !found; ++i) {
                    if (c == lookup_specials[2*i+1]) {
                        s += '\\';
                        s += lookup_specials[2*i];
                        found = true;
                    }
                }
                for(int i = 0; i < 96 && !found; ++i) {
                    if (c && c == (cz ? lookup_cz[i] : lookup_en[i])) {
                        s += i+32;
                        found = true;
                    }
                }
                if (!found) {
                    QString a;
                    if (c&0x40) a.sprintf("\\s\\x%x%x", (c&0x3f)/16, c%16);
                    else a.sprintf("\\x%x%x", c/16, c%16);
                    s += a;
                }
            }
        }
    }
    if (special)
        return "";
    return s;
}
