#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QThread>
#include "protocol.h"

QT_BEGIN_NAMESPACE
namespace Ui { class MainWindow; }
QT_END_NAMESPACE

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    MainWindow(QWidget *parent = 0);
    ~MainWindow();
    Ui::MainWindow *ui;

private slots:
    void on_load_clicked();
    void on_store_clicked();
    void on_info_clicked();
    void on_calibrate_clicked();
    void on_en_clicked();
    void on_cz_clicked();
    void on_setText(const QString text, bool cz);
    void on_error(const QString text);
    void on_warning(const QString text);
    void on_info(const QString text);

private:
    bool loaded;
    void setButtons(bool s);
};

class Thread : public QThread
{
Q_OBJECT
public:
    Thread(MainWindow *mw, void (Thread::*f)(protocol &p));
    ~Thread();
protected:
    void run();
public:
    void read(protocol &p);
    void write(protocol &protocol);
    void information(protocol &protocol);
    void calibrate(protocol &protocol);
private:
    MainWindow *mw;
    void (Thread::*f)(protocol &p);
    int decompress(QByteArray &data, QString &append);
signals:
    void setText(const QString text, bool cz);
    void error(const QString text);
    void warning(const QString text);
    void info(const QString text);
};

#endif // MAINWINDOW_H
