/*
 * Name: buffer.c
 * Project: PassKeychain
 * Author: Jiri Stepanovsky - http://jiristepanovsky.cz
 * Creation Date: 2014-11-28
 * Tabsize: 4
 * License: MIT License
 */


#include "buffer.h"

/* Buffer pointers */
volatile unsigned char bufferRXWritePointer, bufferRXReadPointer, bufferTXWritePointer, bufferTXReadPointer;

/* Actual buffers */
volatile unsigned char rx_buff[RX_SIZE];
volatile unsigned char tx_buff[TX_SIZE];

/* Reset both buffers */
void bufferReset(void){
	bufferRXWritePointer = 0;
	bufferRXReadPointer = 0;
	bufferTXWritePointer = 0;
	bufferTXReadPointer = 0;
}

/* Access to Transmit buffer */
void bufferTXWrite(unsigned char data){
	unsigned char next = bufferTXWritePointer + 1;
	if(next>=(TX_SIZE & 0xFF))
		next = 0;

	if(next != bufferTXReadPointer){
		tx_buff[bufferTXWritePointer] = data;
		bufferTXWritePointer = next;
	} // else { overflow }
}

unsigned char bufferTXRead(void){
	unsigned char data = 0;
	if(bufferTXWritePointer != bufferTXReadPointer){
		data = tx_buff[bufferTXReadPointer];
		bufferTXReadPointer++;
		if(bufferTXReadPointer>=(TX_SIZE&0xFF))
			bufferTXReadPointer = 0;
	}
	return data;
}

unsigned char bufferTXSize(void){
	unsigned char size = 0;
	if(bufferTXWritePointer > bufferTXReadPointer){
		size = bufferTXWritePointer - bufferTXReadPointer;
	} else if (bufferTXWritePointer < bufferTXReadPointer){
		size = (TX_SIZE - bufferTXReadPointer) + bufferTXWritePointer;
	}
	return size;
}

unsigned char bufferTXFree(void){
	unsigned char size = 0;
	if(bufferTXWritePointer >= bufferTXReadPointer){
		size = ((TX_SIZE-1) - bufferTXWritePointer) + bufferTXReadPointer;
	} else if (bufferTXWritePointer < bufferTXReadPointer){
		size = (bufferTXReadPointer - bufferTXWritePointer) - 1;
	}
	return size;
}

/* Access to Receive buffer */
void bufferRXWrite(unsigned char data){
	unsigned char next = bufferRXWritePointer + 1;
	if(next>=(RX_SIZE&0xFF))
		next = 0;

	if(next != bufferRXReadPointer){
		rx_buff[bufferRXWritePointer] = data;
		bufferRXWritePointer = next;
	}
}

unsigned char bufferRXRead(void){
	unsigned char data = 0;
	if(bufferRXWritePointer != bufferRXReadPointer){
		data = rx_buff[bufferRXReadPointer];
		bufferRXReadPointer++;
		if(bufferRXReadPointer>=(RX_SIZE&0xFF))
			bufferRXReadPointer = 0;
	}
	return data;
}

unsigned char bufferRXSize(void){
	unsigned char size = 0;
	if(bufferRXWritePointer > bufferRXReadPointer){
		size = bufferRXWritePointer - bufferRXReadPointer;
	} else if (bufferRXWritePointer < bufferRXReadPointer){
		size = (RX_SIZE - bufferRXReadPointer) + bufferRXWritePointer;
	}
	return size;
}

unsigned char bufferRXFree(void){
	unsigned char size = 0;
	if(bufferRXWritePointer >= bufferRXReadPointer){
		size = ((RX_SIZE-1) - bufferRXWritePointer) + bufferRXReadPointer;
	} else if (bufferRXWritePointer < bufferRXReadPointer){
		size = (bufferRXReadPointer - bufferRXWritePointer) - 1;
	}
	return size;
}
